=== modified file 'plugin/slave/queue_producer.h'
--- plugin/slave/queue_producer.h	2011-03-08 06:28:23 +0000
+++ plugin/slave/queue_producer.h	2011-03-09 17:10:06 +0000
@@ -91,7 +91,7 @@
     _seconds_between_reconnects= seconds;
   }
 
-  void setMaxCommitId(uint64_t value)
+  void setCachedMaxCommitId(uint64_t value)
   {
     _saved_max_commit_id= value;
   }

=== modified file 'plugin/slave/replication_schema.cc'
--- plugin/slave/replication_schema.cc	2011-02-23 16:49:25 +0000
+++ plugin/slave/replication_schema.cc	2011-03-09 17:10:06 +0000
@@ -24,9 +24,11 @@
 #include <drizzled/sql/result_set.h>
 #include <string>
 #include <vector>
+#include <boost/lexical_cast.hpp>
 
 using namespace std;
 using namespace drizzled;
+using namespace boost;
 
 namespace slave
 {
@@ -130,4 +132,15 @@
   return true;
 }
 
+bool ReplicationSchema::setInitialMaxCommitId(uint64_t value)
+{
+  vector<string> sql;
+
+  sql.push_back("UPDATE `sys_replication`.`applier_state`"
+                " SET `last_applied_commit_id` = "
+                + lexical_cast<string>(value));
+
+  return executeSQL(sql);
+}
+
 } /* namespace slave */

=== modified file 'plugin/slave/replication_schema.h'
--- plugin/slave/replication_schema.h	2011-02-17 14:20:50 +0000
+++ plugin/slave/replication_schema.h	2011-03-09 17:10:06 +0000
@@ -33,6 +33,21 @@
   { }
 
   bool create();
+
+  /**
+   * Set initial value of the last applied COMMIT_ID value in applier_state.
+   *
+   * This is used when the server is started with --slave.max-commit-id to
+   * begin reading from the master transaction log at a given point. This
+   * method will persist the value to the applier_state table. If it wasn't
+   * permanently stored immediately, we risk the possibility of losing the
+   * value if the server is again restarted without ever having received
+   * another event from the master (which causes persistence of the value).
+   * An edge case, but still possible.
+   *
+   * @param[in] value The initial value.
+   */
+  bool setInitialMaxCommitId(uint64_t value);
 };
 
 } /* namespace slave */

=== modified file 'plugin/slave/replication_slave.cc'
--- plugin/slave/replication_slave.cc	2011-03-02 16:32:36 +0000
+++ plugin/slave/replication_slave.cc	2011-03-09 17:10:06 +0000
@@ -110,6 +110,15 @@
     return false;
   }
 
+  if (_initial_max_commit_id)
+  {
+    if (not rs.setInitialMaxCommitId(_initial_max_commit_id))
+    {
+      _error= rs.getErrorMessage();
+      return false;
+    }
+  }
+
   return true;
 }
 

=== modified file 'plugin/slave/replication_slave.h'
--- plugin/slave/replication_slave.h	2011-03-07 21:42:32 +0000
+++ plugin/slave/replication_slave.h	2011-03-09 17:10:06 +0000
@@ -41,7 +41,8 @@
 
   ReplicationSlave(const std::string &config)
     : drizzled::plugin::Daemon("Replication Slave"),
-      _config_file(config)
+      _config_file(config),
+      _initial_max_commit_id(0)
   {}
   
   ~ReplicationSlave()
@@ -73,7 +74,10 @@
    */
   void setMaxCommitId(uint64_t value)
   {
-    _producer.setMaxCommitId(value);
+    /* must tell producer to set its cached value */
+    _producer.setCachedMaxCommitId(value);
+    /* setting this indicates that we should store it permanently */
+    _initial_max_commit_id= value;
   }
 
 private:
@@ -89,6 +93,8 @@
   /** I/O thread that will populate the work queue */
   boost::thread _producer_thread;
 
+  uint64_t _initial_max_commit_id;
+
   /**
    * Initialize slave services with the given configuration file.
    *

